#pragma once

#include "logger.h"

namespace cpplogging
{

    namespace __impl
    {
        template<typename T_pred>
        struct stream_setter
        {
            using predicate_type = T_pred;

            predicate_type predicate;
        };

        template<typename T_pred>
        std::ostream& operator<<(std::ostream &os, const stream_setter<T_pred>& x);

        template<typename T_pred>
        constexpr decltype(auto) make_stream_setter(T_pred&& pred);

    }

    /**
     * @brief Set the log level of a log stream.
     */
    inline auto level(log_level level);

    /**
     * @brief Set the sender of a log stream.
     */
    template<typename T_sender>
    inline auto sender(const T_sender* sender);

    /**
     * @brief Set the sender type of a log stream.
     */
    inline auto sender_type(const std::string& sender_type);

    /**
     * @brief Set the file of a log stream.
     */
    inline auto file(const char * file);

    /**
     * @brief Set the line of a log stream.
     */
    inline auto line(int line);

    /**
     * @brief Manipulate the log entry.
     */
    template<typename T_pred>
    inline auto manipulate(T_pred&& pred);

}

#include "iomanip.inl"
