#pragma once

#include <mutex>

#include "consumer.h"

namespace cpplogging
{

    /**
     * @brief Consumer that writes the log entries to a stream,
     */
    struct consumer_stream
        : public consumer
    {
    private:
        using stream_ptr_u = std::unique_ptr<std::ostream>;

    private:
        mutable std::mutex  _mutex;         //!< Mutex to protect the stream.
        std::ostream&       _stream;        //!< Stream to write log entries to.
        stream_ptr_u        _storage;       //!< Pointer to store owned stream.

    public:
        /**
         * @brief Constructor.
         *
         * @param[in] name      Name of the consumer.
         * @param[in] stream    Stream to write data to.
         * @param[in] format    Format to use for logging.
         */
        consumer_stream(
            const std::string& name,
            std::ostream& stream,
            const std::string& format = default_format());

        /**
         * @brief Constructor.
         *
         * @param[in] name      Name of the consumer.
         * @param[in] stream    Stream to write data to.
         * @param[in] format    Format to use for logging.
         */
        consumer_stream(
            const std::string& name,
            stream_ptr_u&& stream,
            const std::string& format = default_format());

        /**
         * @brief Consume a log entry.
         *
         * @param[in] entry Log entry to consume.
         */
        void write_entry(const log_entry_ptr_s& entry) const override;

    };

}
