#pragma once

#include <memory>

#include "../consumer/consumer.h"
#include "../../interface/logger.h"

namespace cpplogging
{

    /**
     * @brief Class to use for matching a rule or a logger.
     */
    struct matcher
    {
        /**
         * @brief Destructor.
         */
        virtual ~matcher() = default;

        /**
         * @brief Check if the passed logger match this matcher.
         *
         * @param[in] logger    Logger to check.
         *
         * @retval true         If the logger matches.
         * @retval false        If the logger does not match.
         */
        virtual bool match(const logger& logger) const;

        /**
         * @brief Check if the passed consumer match this matcher.
         *
         * @param[in] consumer  Consumer to check.
         *
         * @retval true         If the consumer matches.
         * @retval false        If the consumer does not match.
         */
        virtual bool match(const consumer& consumer) const;

    };

    using matcher_ptr_u = std::unique_ptr<matcher>;

}
