#pragma once

#include <regex>

#include "matcher.h"

namespace cpplogging
{

    /**
     * @brief Matcher that uses a regular expression to match a logger or a consumer
     */
    struct matcher_regex
        : public matcher
    {
    private:
        std::regex  _regex;     //!< regex to use for matching
        bool        _invert;    //!< invert the regular expression

    public:
        /**
         * @brief Constructor.
         *
         * @param[in] regex     Regex to use for matching.
         * @param[in] invert    Invert the regular expression.
         */
        matcher_regex(const std::string regex, bool invert = false);

        /**
         * @brief Check if the passed logger match this matcher.
         *
         * @param[in] logger    Logger to check.
         *
         * @retval true         If the logger matches.
         * @retval false        If the logger does not match.
         */
        bool match(const logger& logger) const override;

        /**
         * @brief Check if the passed consumer match this matcher.
         *
         * @param[in] consumer  Consumer to check.
         *
         * @retval true         If the consumer matches.
         * @retval false        If the consumer does not match.
         */
        bool match(const consumer& consumer) const override;
    };

}
