#pragma once

#include "rule.h"

namespace cpplogging
{

    /* rule */

    rule
        ::rule(matcher_ptr_u&& lm, matcher_ptr_u&& cm, log_level min, log_level max)
        : logger_matcher    (std::move(lm))
        , consumer_matcher  (std::move(cm))
        , min_level         (min)
        , max_level         (max)
        { }

    bool rule
        ::is_enabled(log_level level) const
    {
        return min_level <= level
            && max_level >= level;
    }

    void rule
        ::add_consumer(const consumer& consumer)
    {
            std::lock_guard<std::mutex> lg(_mutex);
        _consumer.insert(&consumer);
    }

    void rule
        ::remove_consumer(const consumer& consumer)
    {
        std::lock_guard<std::mutex> lg(_mutex);
        _consumer.erase(&consumer);
    }

    void rule
        ::write_entry(const log_entry_ptr_s& entry) const
    {
        std::lock_guard<std::mutex> lg(_mutex);
        if (is_enabled(entry->level))
        {
            for (auto& c : _consumer)
                c->write_entry(entry);
        }
    }

}
