#include <cpplogging/manager/logger_impl.h>

using namespace ::cpplogging;

void logger_impl::add_rule(rule& rule)
{
    std::lock_guard<std::mutex> lk(_mutex);
    _rules.insert(&rule);
    update_log_level();
}

void logger_impl::remove_rule(rule& rule)
{
    std::lock_guard<std::mutex> lk(_mutex);
    _rules.erase(&rule);
    update_log_level();
}

bool logger_impl::is_enabled(log_level level) const
{
    return _min_level <= level
        && _max_level >= level;
}

void logger_impl::write_entry(const log_entry_ptr_s& entry) const
{
    std::lock_guard<std::mutex> lk(_mutex);

    if (!is_enabled(entry->level))
        return;

    for (auto& r : _rules)
        r->write_entry(entry);
}

void logger_impl::update_log_level()
{
    _min_level = log_level::unknown;
    _max_level = log_level::unknown;

    for (auto& r : _rules)
    {
        if (    _min_level == log_level::unknown
            ||  _min_level > r->min_level)
            _min_level = r->min_level;

        if (    _max_level == log_level::unknown
            ||  _max_level < r->max_level)
            _max_level = r->max_level;
    }
}
